/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.vipold.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.vipold.entity.BizVipOld;
import vip.xiaonuo.biz.modular.vipold.param.BizVipOldAddParam;
import vip.xiaonuo.biz.modular.vipold.param.BizVipOldEditParam;
import vip.xiaonuo.biz.modular.vipold.param.BizVipOldIdParam;
import vip.xiaonuo.biz.modular.vipold.param.BizVipOldPageParam;
import vip.xiaonuo.biz.modular.vipold.service.BizVipOldService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 旧版会员控制器
 *
 * @author 徐超
 * @date  2025/06/26 22:07
 */
@Tag(name = "旧版会员控制器")
@RestController
@Validated
public class BizVipOldController {

    @Resource
    private BizVipOldService bizVipOldService;

    /**
     * 获取旧版会员分页
     *
     * @author 徐超
     * @date  2025/06/26 22:07
     */
    @Operation(summary = "获取旧版会员分页")
    @SaCheckPermission("/biz/vipold/page")
    @GetMapping("/biz/vipold/page")
    public CommonResult<Page<BizVipOld>> page(BizVipOldPageParam bizVipOldPageParam) {
        return CommonResult.data(bizVipOldService.page(bizVipOldPageParam));
    }

    /**
     * 添加旧版会员
     *
     * @author 徐超
     * @date  2025/06/26 22:07
     */
    @Operation(summary = "添加旧版会员")
    @CommonLog("添加旧版会员")
    @SaCheckPermission("/biz/vipold/add")
    @PostMapping("/biz/vipold/add")
    public CommonResult<String> add(@RequestBody @Valid BizVipOldAddParam bizVipOldAddParam) {
        bizVipOldService.add(bizVipOldAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑旧版会员
     *
     * @author 徐超
     * @date  2025/06/26 22:07
     */
    @Operation(summary = "编辑旧版会员")
    @CommonLog("编辑旧版会员")
    @SaCheckPermission("/biz/vipold/edit")
    @PostMapping("/biz/vipold/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizVipOldEditParam bizVipOldEditParam) {
        bizVipOldService.edit(bizVipOldEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除旧版会员
     *
     * @author 徐超
     * @date  2025/06/26 22:07
     */
    @Operation(summary = "删除旧版会员")
    @CommonLog("删除旧版会员")
    @SaCheckPermission("/biz/vipold/delete")
    @PostMapping("/biz/vipold/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<BizVipOldIdParam> bizVipOldIdParamList) {
        bizVipOldService.delete(bizVipOldIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取旧版会员详情
     *
     * @author 徐超
     * @date  2025/06/26 22:07
     */
    @Operation(summary = "获取旧版会员详情")
    @SaCheckPermission("/biz/vipold/detail")
    @GetMapping("/biz/vipold/detail")
    public CommonResult<BizVipOld> detail(@Valid BizVipOldIdParam bizVipOldIdParam) {
        return CommonResult.data(bizVipOldService.detail(bizVipOldIdParam));
    }
}
