/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.vipold.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.vipold.entity.BizVipOld;
import vip.xiaonuo.biz.modular.vipold.mapper.BizVipOldMapper;
import vip.xiaonuo.biz.modular.vipold.param.BizVipOldAddParam;
import vip.xiaonuo.biz.modular.vipold.param.BizVipOldEditParam;
import vip.xiaonuo.biz.modular.vipold.param.BizVipOldIdParam;
import vip.xiaonuo.biz.modular.vipold.param.BizVipOldPageParam;
import vip.xiaonuo.biz.modular.vipold.service.BizVipOldService;

import java.util.List;

/**
 * 旧版会员Service接口实现类
 *
 * @author 徐超
 * @date  2025/06/26 22:07
 **/
@Service
public class BizVipOldServiceImpl extends ServiceImpl<BizVipOldMapper, BizVipOld> implements BizVipOldService {

    @Override
    public Page<BizVipOld> page(BizVipOldPageParam bizVipOldPageParam) {
        QueryWrapper<BizVipOld> queryWrapper = new QueryWrapper<BizVipOld>().checkSqlInjection();
        if(ObjectUtil.isNotEmpty(bizVipOldPageParam.getPhone())) {
            queryWrapper.lambda().like(BizVipOld::getPhone, bizVipOldPageParam.getPhone());
        }
        if(ObjectUtil.isNotEmpty(bizVipOldPageParam.getName())) {
            queryWrapper.lambda().like(BizVipOld::getName, bizVipOldPageParam.getName());
        }
        //if(ObjectUtil.isAllNotEmpty(bizVipOldPageParam.getSortField(), bizVipOldPageParam.getSortOrder())) {
        //    CommonSortOrderEnum.validate(bizVipOldPageParam.getSortOrder());
        //    queryWrapper.orderBy(true, bizVipOldPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
        //            StrUtil.toUnderlineCase(bizVipOldPageParam.getSortField()));
        //} else {
        //    queryWrapper.lambda().orderByAsc(BizVipOld::getId);
        //}
        queryWrapper.lambda().orderByDesc(BizVipOld::getCardTime);
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(BizVipOldAddParam bizVipOldAddParam) {
        BizVipOld bizVipOld = BeanUtil.toBean(bizVipOldAddParam, BizVipOld.class);
        this.save(bizVipOld);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(BizVipOldEditParam bizVipOldEditParam) {
        BizVipOld bizVipOld = this.queryEntity(bizVipOldEditParam.getId());
        BeanUtil.copyProperties(bizVipOldEditParam, bizVipOld);
        this.updateById(bizVipOld);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<BizVipOldIdParam> bizVipOldIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(bizVipOldIdParamList, BizVipOldIdParam::getId));
    }

    @Override
    public BizVipOld detail(BizVipOldIdParam bizVipOldIdParam) {
        return this.queryEntity(bizVipOldIdParam.getId());
    }

    @Override
    public BizVipOld queryEntity(String id) {
        BizVipOld bizVipOld = this.getById(id);
        if(ObjectUtil.isEmpty(bizVipOld)) {
            throw new CommonException("旧版会员不存在，id值为：{}", id);
        }
        return bizVipOld;
    }
}
